library ArcNeuralNetwork.DataSet;

import "instance.dart";


class DataSet {

  String title;
  List<Instance> instances;

  DataSet(this.title) {
    this.instances = [];
  }

  DataSet.FromList(this.title, List<List<double>> instances, List<String> labels, [List<int> classValue]) {
    this.instances = [];
    if (classValue != null) {
      for (var instance in instances) {
        this.instances.add(new Instance.PairValues(instance, labels, classValue));
      }
    } else {
      for (var instance in instances) {
        this.instances.add(new Instance.PairValues(instance, labels));
      }
    }
  }

  //////////////////////////////////NUMBER OF///////////////////////////////////////

  get numberInstances => this.instances.length;
  get numClassValues => this.instances[0].numClassValues;
  get numNonClassValues => this.instances[0].numNonClassValues;
  get numValues => this.instances[0].numValues;

  ////////////////////////////////// GETTERS ///////////////////////////////////////

  get trainSet {
    List<Instance> trainSet = [];
    for (var i = 0; i < this.instances; i++) {
      if (this.instances[i].isForTrain) {
        trainSet.add(this.instances[i]);
      }
    }
    return trainSet;
  }
  get testSet {
    List<Instance> testSet = [];
    for (var i = 0; i < this.instances; i++) {
      if (!this.instances[i].isForTrain) {
        testSet.add(this.instances[i]);
      }
    }
    return testSet;
  }

  get trainSetValues {
    List<Instance> trainSet = this.trainSet;
    List<List<double>> Value = [];
    for (Instance instance in trainSet) {
      Value.add(instance.pairsValues);
    }
    return Value;
  }

  get testSetValues {
    List<Instance> testSet = this.testSet;
    List<List<double>> Value = [];
    for (Instance instance in testSet) {
      Value.add(instance.pairsValues);
    }
    return Value;
  }

  get dataSetValues {
    List<List<double>> Value = this.trainSetValues;
    Value.addAll(this.testSetValues);
    return Value;
  }

  get trainSetClassValues {
    List<Instance> trainSet = this.trainSet;
    List<List<double>> classValue = [];
    for (Instance instance in trainSet) {
      classValue.add(instance.pairsClassValues);
    }
    return classValue;
  }

  get testSetClassValues {
    List<Instance> testSet = this.testSet;
    List<List<double>> classValue = [];
    for (Instance instance in testSet) {
      classValue.add(instance.pairsClassValues);
    }
    return classValue;
  }

  get instancesClassValues {
    List<List<double>> classValue = this.trainSetClassValues;
    classValue.addAll(this.testSetClassValues);
    return classValue;
  }

  get trainSetNonClassValues {
    List<Instance> trainSet = this.trainSet;
    List<List<double>> nonClassValue = [];
    for (Instance instance in trainSet) {
      nonClassValue.add(instance.pairsNonClassValues);
    }
    return nonClassValue;
  }

  get testSetNonClassValues {
    List<Instance> testSet = this.testSet;
    List<List<double>> nonClassValue = [];
    for (Instance instance in testSet) {
      nonClassValue.add(instance.pairsNonClassValues);
    }
    return nonClassValue;
  }

  get instancesNonClassValues {
    List<List<double>> nonClassValue = this.trainSetClassValues;
    nonClassValue.addAll(this.testSetClassValues);
    return nonClassValue;
  }

  List<double> valuesOfLabel(String label) {
    List<double> values = [];
    for (var instance in this.instances) {
      values.add(instance.valueOfLabel(label));
    }
    return values;
  }

  set trainInstancesPercentage(int percentage) {
    int lengthTrainSet = ((this.instances.length * percentage) / 100).round();
    int lengthTestSet = this.instances.length - lengthTrainSet;
    for (var i = 0; i < lengthTrainSet; i++) {
      this.instances[i].train = true;
    }
    for (var i = testSet; this.instances.length; i++) {
      this.instances[i].train = false;
    }

  }

  set trainInstancesNumber(int number) {
    if (number > this.instances.length) {
      throw ("Index out of bounds");
    }
    for (var i = 0; i < number; i++) {
      this.instances[i].train = true;
    }
    for (var i = number; i < this.instances.length; i++) {
      this.instances[i].train = false;
    }
  }


  Instance instanceAt(int index) {
    return this.instances[index];
  }

  void addInstance(Instance instance) {
    this.instances.add(instance);
  }

  void addInstanceAt(int index, Instance instance) {
    this.instances.insert(index, instance);
  }

  void removeInstance(Instance instance) {
    this.instances.remove(instance);
  }

  void removeInstanceAt(int index) {
    this.instances.removeAt(index);
  }

}
