library ArcNeuralNetwork.DataSet.Instance;

import 'pair.dart';

class Instance {

  List<Pair> pairs;
  bool train;

  //Create a default instance

  Instance() {
    this.pairs = [];
  }

  //Crate a instance with a list of values and labels. Optional index list to set classValues

  Instance.PairValues(List<double> values, List<String> labels, [List<int> indexClassValue]) {
    if (labels.length != values.length) {
      throw ("Must be the same number of labels and values");
    }

    this.pairs = [];

    if (indexClassValue != null) {
      for (var i = 0; i < labels; i++) {
        if (indexClassValue.contains(i)) {
          this.pairs.add(new Pair(values[i], labels[i], true));
        } else {
          this.pairs.add(new Pair(values[i], labels[i], false));
        }
      }
    } else {
      for (var i = 0; i < labels; i++) {
        this.pairs.add(new Pair(values[i], labels[i]));
      }
    }
  }

  //Create a instance with a list of pair attribute-value

  Instance.Pair(this.pairs);



  ////////////////////////////////// NUMBER OF //////////////////////////////////

  get numClassValues {
    int count = 0;
    for (var pair in this.pairs) {
      if (pair.classValue) {
        count++;
      }
    }
    return count;
  }

  get numNonClassValues {
    return this.pairs.length - this.numClassValues;
  }

  get numValues {
    return this.pairs.length;
  }

  ////////////////////////////// ELEMENT AT //////////////////////////////////////

  Pair pairAt(int index) {
    return this.pairs[index];
  }

  double pairValueAt(int index) {
    return this.pairs[index].value;
  }

  String pairLabelAt(int index) {
    return this.pairs[index].label;
  }

  bool pairClassValueAt(int index) {
    return this.pairs[index].classValue;
  }

  double valueOfLabel(String label) {
    double value = 0.0;
    if (!this.labelExist(label)) {
      throw ("Label does not exist");
    }
    for (var pair in this.pairs) {
      if (pair.label == label) {
        value = pair.value;
      }
    }
    return value;
  }

  bool labelExist(String label) {
    bool exist = false;
    for (var pair in this.pairs) {
      if (pair.label == label) {
        exist = true;
      }
    }
    return exist;
  }

  ///////////////////////////////////// GETTERS /////////////////////////////////////////

  get pairsIndexClassValues {
    List<int> index = [];
    for (var i = 0; i < this.pairs.length; i++) {
      if (this.pairs[i].classValue) {
        index.add(i);
      }
    }
    return index;
  }

  get pairsIndexNonClassValues {
    List<int> index = [];
    for (var i = 0; i < this.pairs.length; i++) {
      if (!this.pairs[i].classValue) {
        index.add(i);
      }
    }
    return index;
  }

  get pairsClassValues {
    List<Pair> classValues = [];
    for (var pair in this.pairs) {
      if (pair.classValue) {
        classValues.add(pair);
      }
    }
    return classValues;
  }

  get pairsNonClassValues {
    List<Pair> nonClassValues = [];
    for (var pair in this.pairs) {
      if (!pair.classValue) {
        nonClassValues.add(pair);
      }
    }
    return nonClassValues;
  }

  get pairsLabels {
    List<String> labels = [];
    for (var pair in this.pairs) {
      labels.add(pair.label);
    }
    return labels;
  }

  get pairsValues {
    List<double> values = [];
    for (var pair in this.pairs) {
      values.add(pair.value);
    }
    return values;
  }

  get hasPairs => this.pairs.isEmpty;

  get hasClassValues {
    bool has = false;
    for (var pair in pairs) {
      if (pair.classValue) {
        has = true;
      }
    }
    return has;
  }

  get isForTrain => this.train;

  get isForTest {
    if (this.train == true) {
      return false;
    } else {
      return true;
    }
  }

  /////////////////////////////////// SETTERS /////////////////////////////////////////

  void setPair(int index, Pair pair) {
    this.pairs[index] = pair;
  }

  void setPairParameters(int index, [double value, String label, bool classValue]) {
    if (value != null) {
      this.pairs[index].value = value;
    }
    if (label != null) {
      this.pairs[index].label = label;
    }
    if (classValue != null) {
      this.pairs[index].classValue = classValue;
    }
  }

  ////////////////////////////////////// Add a pair//////////////////////////////////////////////



  //Create a class value pair and inserts in the list. It´s possible to set a position for the new class value

  void addPair(Pair pair, [int index]) {

    if (index != null) {
      this.pairs.insert(index, pair);
    } else {
      this.pairs.add(pair);
    }
  }

  //Create a class value pair and inserts in the list. It´s possible to set a position for the new class value

  void addPairWithValue(double value, String label, [int index]) {

    if (index != null) {
      this.pairs.insert(index, new Pair(value, label));
    } else {
      this.pairs.add(new Pair(value, label));
    }
  }

  //////////////////////////////////// DELETE PAIR /////////////////////////////////////

  void deletePairAt(int index) {
    this.pairs.removeAt(index);
  }

  void deletePair(Pair pair) {
    this.pairs.remove(pair);
  }
}
