library ArcNeuralNetwork.Filter;

import "dart:math";
import '../Dataset/DataSet.dart';

abstract class Filter {

  String name;
  String description;

  DataSet applyFilter(DataSet dataSet);

}

class RandomizeFilter extends Filter {

  //Constructor

  RandomizeFilter() {

    this.name = "Randomize";
    this.description = "Randomize the instance order of the dataset";
  }

  //Method filter

  DataSet applyFilter(DataSet input) {

    List items = input.instances;
    var random = new Random();

    // Go through all elements.
    for (var i = items.length - 1; i > 0; i--) {

      // Pick a pseudorandom number according to the list length
      var n = random.nextInt(items.length);

      var temp = items[i];
      items[i] = items[n];
      items[n] = temp;
    }

    input.instances = items;
    return input;
  }

}

class NormalizationFilter extends Filter {


  NormalizationFilter() {
    this.name = "Normailization";
    this.description = "Instance values between 0 and 1";
  }

  DataSet applyFilter(DataSet input) {
    List items = transposedMatrix(input.dataSetValues);    
    for (int i = 0; i < items.length; i++) {
      items[i] = normalizationColumn(items[i]);
    }
    items = transposedMatrix(items);
    input.instances = items;
    return input;
  }

  List transposedMatrix(List<List> original) {
    List<List> matrixTransposed = [];
    for (int i = 0; i < original.length; i++) {
      for (int j = 0; j < original[i].length; j++) {
        matrixTransposed[i].add(original[j][i]);
      }
    }
    return matrixTransposed;
  }

  List normalizationColumn(List column) {
    num maxim = column.reduce(max);
    num minim = column.reduce(min);
    List columnNormalized = [];
    for (num i = 0; i < column.length; i++) {
      columnNormalized.add((column[i] - minim) / maxim - minim);
    }
    return columnNormalized;
  }


}
