library ArcNeuralNetwork.ActivationFunction;

//Imports

import "dart:math";

/*
Class InputFunction: represent the activation function to calculate the output value of a neuron
*/

abstract class ActivationFunction {
  double getOutput(double value);
}

/*
  Class Step: extends ActivationFunction.
    Methods:
      - get output: step function

 */

class Step extends ActivationFunction {
  double getOutput(double value) {
    if (value >= 0) {
      return 1.0;
    } else {
      return -1.0;
    }
  }
}

/*
  Class Gaussian: extends ActivationFunction.
    Methods:
      - get output: gaussian function

 */

class Gaussian extends ActivationFunction {
  double sigma = 0.5;

  double getOutput(double value) {
    return exp(-pow(value, 2) / (2 * pow(this.sigma, 2)));
  }
}

/*
  Class Sigmoid: extends ActivationFunction.
    Methods:
      - get output: sigmoid function

 */

class Sigmoid extends ActivationFunction {
  double getOutput(double value) {
    return (1 / (1 + pow(E, (-1 * value))));
  }
}

/*
  Class Tanh: extends ActivationFunction.
    Methods:
      - get output: tanh function

 */

class Tanh extends ActivationFunction {
  double getOutput(double value) {
    return ((pow(E, value) - pow(E, -1 * value)) / (pow(E, value) + pow(E, -1 * value)));
  }
}

/*
  Class Cosh: extends ActivationFunction.
    Methods:
      - get output: cosh function

 */

class Cosh extends ActivationFunction {
  double getOutput(double value) {
    return (pow(E, value) + pow(E, -1 * value)) / 2;
  }

}

/*
  Class Sinh: extends ActivationFunction.
    Methods:
      - get output: sinh function

 */

class Sinh extends ActivationFunction {
  double getOutput(double value) {
    return (pow(E, value) - pow(E, -1 * value)) / 2;
  }
}

/*
  Class Linear: extends ActivationFunction.
    Methods:
      - get output: linear function

 */

class Linear extends ActivationFunction {
  double getOutput(double value) {
    return value;
  }

}

/*
  Class Log: extends ActivationFunction.
    Methods:
      - get output: log function

 */

class Log extends ActivationFunction {
  double getOutput(double value) {
    return log(value);
  }

}
