library arcNeuralNetwork.InputFunction;

//Imports

import "dart:math";
import "../neuron.dart";

/*
Class InputFunction: represent a input function to calculate the input value of a neuron in base of its connections
*/

abstract class InputFunction {
  double getOutput(List<Connection> inputConnections);
}

/*
  Class Weight Combination: extends InputFunction.
    Methods:
      - get output: If X is the vector of entries and W is the vector of weight of the input connections the total input is:

                                      E += (Xi * Wi)
 */

class WeightCombination extends InputFunction {

  double getOutput(List<Connection> inputConnections) {
    double output = 0.0;
    for (var i in inputConnections) {
      output += i.inputValueConnection * i.weightValueConnection;
    }
    return output;
  }
}

/*
  Class Max: extends InputFunction.
    Methods:
      - get output: If X is the vector of entries and W is the vector of weight of the input connections the total input is:

                                      E = max(Xi * Wi)
 */

class Max extends InputFunction {
  double getOutput(List<Connection> inputConnections) {
    List<double> output = [];
    for (var i in inputConnections) {
      output.add(i.inputValueConnection * i.weightValueConnection);
    }
    return output.reduce(max);
  }
}

/*
  Class Min: extends InputFunction.
    Methods:
      - get output: If X is the vector of entries and W is the vector of weight of the input connections the total input is:

                                      E = min(Xi * Wi)
 */

class Min extends InputFunction {
  double getOutput(List<Connection> inputConnections) {
    List<double> output = [];
    for (var i in inputConnections) {
      output.add(i.inputValueConnection * i.weightValueConnection);
    }
    return output.reduce(min);
  }
}
