library ArcNeuralNetwork.Network;

//Imports

import "neuron.dart";
import "layer.dart";
import 'LearningRule/learningrule.dart';

/*
Class Network: represent the neural network
      Parameters:
        - Layers: all the layers that forms the network --> List<Layer>
        - Output: all the outputs of the network --> List<double>
        - Input Neuron: the input neurons of the network --> List<neuron>
        - Output Neuron: the output neurons of the network --> List<neuron>
        - Learning Rule: of the network --> LearningRule
      Methods:
        - Add Layer
        - Remove Layer
        - Inputs and Outputs: methods to set input neurons and output neurons of the network
        - Calculate Output: calculate the output of the network
        - Get weights
        - Set weights
        - Check Layers
        - Create Connection
*/

class Network {

  List<Layer> layers;
  List<double> output;
  List<Neuron> inputNeuron;
  List<Neuron> outputNeuron;
  LearningRule learningRule;

  Network() {
    this.layers = [];
    this.inputNeuron = [];
    this.outputNeurons = [];
  }

  ///////////////////////////////////  Add Layer  /////////////////////////////////

  void addLayer(Layer layer) {
    this.layers.add(layer);
    layer.network = this;
  }

  void addLayerIndex(int index, Layer layer) {
    this.layers.insert(index, layer);
    layer.network = this;
  }

  ///////////////////////////////////  Remove Layer  /////////////////////////////////

  void removeLayer(Layer layer) {
    this.layers.remove(layer);
  }

  void removeLayerIndex(int index) {
    this.layers.removeAt(index);
  }

  void removeAllLayers() {
    this.layers = [];
  }

  ///////////////////////////////////  Inputs and Outputs  /////////////////////////////////

  set inputNetwork(List<double> inputs) {
    if (inputs.length != this.inputNeuron.length) {
      throw ("Array length distinct");
    }
    for (var i = 0; i < this.inputNeuron.length; i++) {
      this.inputNeuron[i].input = inputs[i];
    }
  }
  List<double> get outputNetwork {
    // double[] outputVector = new double[outputNeurons.length];// use attribute to avoid creating to arrays and avoid GC work
    for (int i = 0; i < this.outputNeuron.length; i++) {
      output[i] = this.outputNeuron[i].output;
    }

    return output;
  }

  set outputNeurons(List<Neuron> outputNeurons) {
    for (Neuron neuron in outputNeurons) {
      this.outputNeuron.add(neuron);
    }
    this.output = [outputNeurons.length];
  }

  set outputLabels(List<String> ids) {
    for (int i = 0; i < this.outputNeuron.length; i++) {
      this.outputNeuron[i].id = ids[i];
    }
  }

  ///////////////////////////////////  Output network calculate  /////////////////////////////////

  void calculateOutput() {
    for (Layer layer in this.layers) {
      layer.calculateOutput();
    }
  }

  ///////////////////////////////////  Get weights  /////////////////////////////////

  List get weights {
    List weights = [];
    for (Layer layer in this.layers) {
      weights.add(layer.weights);
    }
    return weights;
  }

  List get weightsValues {
    List weights = [];
    for (Layer layer in this.layers) {
      weights.add(layer.weightsValues);
    }
    return weights;
  }

  List get weightsPreviousValues {
    List weights = [];
    for (Layer layer in this.layers) {
      weights.add(layer.weightsPreviousValues);
    }
    return weights;
  }

  ///////////////////////////////////  Set weights  /////////////////////////////////

  void weightsValue(double value) {
    for (Layer layer in this.layers) {
      layer.setAllWeightValue(value);
    }
  }

  void setWeightsRandomValue([double rangeMin, double rangeMax]) {
    for (Layer layer in this.layers) {
      layer.setAllWeightValueRandom(rangeMin, rangeMax);
    }
  }

  ///////////////////////////////////  Check Layers  /////////////////////////////////

  bool hasLayers() {
    return this.layers.isEmpty;
  }

  ///////////////////////////////////  Connection  /////////////////////////////////

  void createConnection(Neuron from, Neuron to, double weightVal) {
    to.addInputConnection(new Connection.WeightValue(from, to, weightVal));
  }


}
